import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:syncrow_web/pages/common/bloc/project_cubit.dart';
import 'package:syncrow_web/pages/spaces_management/space_model/bloc/space_model_event.dart';
import 'package:syncrow_web/pages/spaces_management/space_model/bloc/space_model_state.dart';
import 'package:syncrow_web/pages/spaces_management/space_model/models/space_template_model.dart';
import 'package:syncrow_web/services/space_model_mang_api.dart';
import 'package:syncrow_web/utils/constants/temp_const.dart';

class SpaceModelBloc extends Bloc<SpaceModelEvent, SpaceModelState> {
  final SpaceModelManagementApi api;
  final ProjectCubit projectCubit;

  SpaceModelBloc({
    required this.api,
    required this.projectCubit,
    required List<SpaceTemplateModel> initialSpaceModels,
  }) : super(SpaceModelLoaded(spaceModels: initialSpaceModels)) {
    on<CreateSpaceModel>(_onCreateSpaceModel);
    on<UpdateSpaceModel>(_onUpdateSpaceModel);
  }

  Future<void> _onCreateSpaceModel(
      CreateSpaceModel event, Emitter<SpaceModelState> emit) async {
    final currentState = state;

    if (currentState is SpaceModelLoaded) {
      try {
        final projectUuid = projectCubit.state;
        final newSpaceModel = await api.getSpaceModel(
            event.newSpaceModel.uuid ?? '', projectUuid ?? TempConst.projectId);

        if (newSpaceModel != null) {
          final updatedSpaceModels =
              List<SpaceTemplateModel>.from(currentState.spaceModels)
                ..add(newSpaceModel);
          emit(SpaceModelLoaded(spaceModels: updatedSpaceModels));
        }
      } catch (e) {
        emit(SpaceModelError(message: e.toString()));
      }
    }
  }

  Future<void> _onUpdateSpaceModel(
      UpdateSpaceModel event, Emitter<SpaceModelState> emit) async {
    final currentState = state;
    if (currentState is SpaceModelLoaded) {
      try {
        final projectUuid = projectCubit.state;

        final newSpaceModel = await api.getSpaceModel(
            event.spaceModelUuid ?? '', projectUuid ?? TempConst.projectId);
        if (newSpaceModel != null) {
          final updatedSpaceModels = currentState.spaceModels.map((model) {
            return model.uuid == event.spaceModelUuid ? newSpaceModel : model;
          }).toList();
          emit(SpaceModelLoaded(spaceModels: updatedSpaceModels));
        }
      } catch (e) {
        emit(SpaceModelError(message: e.toString()));
      }
    }
  }
}
